from .db import DB
from os.path import join
from .config import (
    base_folder,
    create_folder,
    get_tables,
    create_spark_session,
    init_arquee_db,
)
from .config import logger
from py4j.protocol import Py4JJavaError


class Arquee:
    def __init__(
        self,
        origin_db: str = "ORIGIN_DB",
        destiny_db: str = "DESTINY_DB",
        tables_file: str = "tables.txt",
    ):
        self.origin_db = DB(origin_db)
        self.destiny_db = DB(destiny_db)

        logger.info(f"{origin_db} connected successfully")
        logger.info(f"{destiny_db} connected successfully")

        self.datalake = join(base_folder, "data", "datalake")
        self.warehouse = join(base_folder, "data", "warehouse")

        create_folder(self.datalake)
        create_folder(self.warehouse)

        logger.info("Datalake folder created successfully")
        logger.info("Warehouse folder created successfully")

        self.tables = get_tables(tables_file)

        logger.info(f"{tables_file} processed successfully")

        self.history_db_cursor = init_arquee_db()
        

    def create_datalake(self):
        spark = create_spark_session()
        self.spark = spark

        for table in self.tables:
            try:
                table_path = join(self.datalake, table)
                df = spark.read.jdbc(self.origin_db.url, table)
                df.write.parquet(table_path, mode="overwrite")
                logger.info(f"Table {table} loaded successfully on datalake")

            except Py4JJavaError:
                logger.error(f"{table} not exist on db {self.origin_db.db_name}")
                continue
