from os.path import join, exists
from os import makedirs
from shutil import rmtree
from pyspark.sql import SparkSession
from datetime import datetime
import logging
from dotenv import load_dotenv
from sqlite3 import connect, Cursor, OperationalError
from time import time

load_dotenv()

execution_id = str(time())
base_folder = "arquee"
logs_folder = join(base_folder, "logs")
logs_file = datetime.now().strftime("%d-%m-%y--%H,%M")
logs_file = join(logs_folder, f"{logs_file}.log")


try:
    makedirs(logs_folder)
except Exception as e:
    pass

FORMAT = "%(asctime)s - %(levelname)s - %(message)s"

logging.basicConfig(filename=logs_file, level=logging.INFO, format=FORMAT)

logger = logging.getLogger()


def arquee_error(message: str):
    logger.error(message)
    print("ERROR, EXITED")
    exit(1)


def create_folder(folder_name: str):
    try:
        if exists(folder_name):
            rmtree(folder_name)
            makedirs(folder_name)
        else:
            makedirs(folder_name)

    except Exception as e:
        print("e")
        arquee_error(e)


def init_arquee_db() -> Cursor:
    arquee_db_path = join(logs_folder, "arquee.db")
    cursor_history_db = connect(arquee_db_path).cursor()
    try:
        cursor_history_db.execute(
            "CREATE TABLE HISTORY(id,timestamp,status,file_location)"
        )
        logger.info("HISTORY DB created successfully")

        cursor_history_db.execute(
            f"""INSERT INTO HISTORY(id,timestamp,status,file_location)
            VALUES({execution_id}, {execution_id}, {logs_file}, FAILED)"""
        )
    except OperationalError:
        logger.info("HISTORY DB already exists, skiping creation")
        pass


def get_tables(tables_file: str) -> list[str]:
    try:
        with open(tables_file, "r") as file:
            tables = file.readlines()
            return [table.strip() for table in tables]
    except FileNotFoundError:
        arquee_error(f"File {tables_file} not found")


def create_spark_session() -> SparkSession:
    spark: SparkSession = (
        SparkSession.builder.appName("ETL")
        .master("local[*]")
        .config("spark.executor.memory", "4g")
        .config("spark.sql.parquet.int96RebaseModeInWrite", "CORRECTED")
        .getOrCreate()
    )
    return spark
